<?php
namespace Auction\Entity;

use Doctrine\ORM\Mapping as ORM;



/**
 * Notification logs.
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 * @ORM\Table(name="auction_public_holiday")
 */
class PublicHoliday
{

	/**
	 * Can archive records.
	 */
	const ARCHIVE = false;
	/**
	 * Pull Synchronization Strategy for this table.
	 */
	const PULL_SYNCH_STRATEGY = false;
	/**
	 * Push Synchronization Strategy for this table.
	 */
	const PUSH_SYNCH_STRATEGY = false;

	/**
	 * @ORM\Id
	 * @ORM\Column(type="integer");
	 * @ORM\GeneratedValue(strategy="AUTO")
	 */
	protected $id;

	/**
	 * @ORM\Column(type="string", nullable=true, length=50)
	 */
	protected $name;

	/**
	 * @ORM\Column(type="date");
	 */
	protected $holiday;

	/**
	 * @ORM\Column(type="datetime");
	 */
	protected $created;

	/**
	 * @ORM\Column(type="datetime", nullable=true);
	 */
	protected $updated;


	/**
	 * Magic getter to expose protected properties.
	 * @param string $property
	 * @return mixed
	 */
	public function __get($property)
	{
		return $this->$property;
	}

	/**
	 * Magic setter to save protected properties.
	 * @param string $property
	 * @param mixed $value
	 */
	public function __set($property, $value)
	{
		$this->$property = $value;
	}

	/**
	 * @ORM\PrePersist
	 */
	public function setCreateTime()
	{
		$this->created = new \DateTime("now");
	}

	/**
	 * @ORM\PreUpdate
	 */
	public function setUpdateTime()
	{
		$this->updated = new \DateTime("now");
	}

	/**
	 * Convert the object to an array.
	 * @param array $expand
	 * @param array $intersect
	 * @return array
	 */
	public function toArray(array $expand = array('attachment'), array $intersect = array())
	{
		$dateTimeFormat = \Utility\Registry::getConfigParam('DateTimeFormat');
		$dateFormat     = \Utility\Registry::getConfigParam('DateFormat');
		$includeAll     = empty($intersect);
		$data = array();
		($includeAll || isset($intersect['id']))
			&& $data['id'] = $this->id;
		($includeAll || isset($intersect['name']))
			&& $data['name'] = $this->name;
		($includeAll || isset($intersect['holiday']))
			&& $data['holiday'] = !is_null($this->holiday)
				? $this->holiday->format($dateFormat)
				: null;
		($includeAll || isset($intersect['created']))
			&& $data['created'] = !is_null($this->created)
				? $this->created->format($dateTimeFormat)
				: null;
		($includeAll || isset($intersect['updated']))
			&& $data['updated'] = !is_null($this->updated)
				? $this->updated->format($dateTimeFormat)
				: null;
		return $data;
	}

	/**
	 * Populate from an array.
	 * @param array $data
	 */
	public function fromArray($data = array())
	{
		isset($data['id'])
			&& $this->id = $data['id'];
		isset($data['name'])
			&& $this->name = $data['name'];
		isset($data['holiday'])
			&& $this->holiday = is_object($data['holiday'])
			? $data['holiday']
			: new \DateTime($data['holiday']);
	}

}
